@@ -10,7 +10,8 @@
 #' Gibbs samples that shall be regarded as burnin.
 #' @param thin An integer to set the thinning interval range. If thin = 1,
 #' every iteration of the Gibbs-sampling chain will be kept. For highly autocorrelated
-#' chains, that are only examined by few iterations (say less than 1000),
+#' chains, that are only examined by few iterations (say less than 1000).
+#' @param rounding_degrees A numeric vector with the presumed rounding degrees.
 #' @return A list with 1. 'y_ret' the n x 1 data.frame with the original and imputed values.
 #' 2. 'Sol' the Gibbs-samples for the fixed effects parameters.
 #' 3. 'VCV' the Gibbs-samples for variance parameters.
@@ -20,7 +21,8 @@ imp_orderedcat_multi <- function(y_imp,
                       clID,
                       nitt = 25000,
                       burnin = 5000,
-                      thin = 20){
+                      thin = 20,
+                      rounding_degrees = c(1, 10, 100, 1000)){
 
   # ----- checking input
   if(!is.factor(y_imp)){
@@ -30,9 +32,9 @@ imp_orderedcat_multi <- function(y_imp,
 
   # -----------------------------preparing the data ------------------
   X_imp <- cleanup(X_imp)
-  X_imp_stand <- stand(X_imp)
+  X_imp_stand <- stand(X_imp, rounding_degrees = rounding_degrees)
   Z_imp <- cleanup(Z_imp)
-  Z_imp_stand <- stand(Z_imp)
+  Z_imp_stand <- stand(Z_imp, rounding_degrees = rounding_degrees)
 
   ph <- sample_imp(y_imp)[, 1]
 
@@ -67,7 +69,7 @@ imp_orderedcat_multi <- function(y_imp,
   # -------- better model to remove insignificant variables
 
   types <- array(dim = ncol(tmp_0_all))
-  for(i in 1:length(types)) types[i] <- get_type(tmp_0_all[, i])
+  for(i in 1:length(types)) types[i] <- get_type(tmp_0_all[, i], rounding_degrees = rounding_degrees)
   #polr needs an intercept variable. So we have to include one to the model formula,
   #but not if the data already have an intercept variable.
 
