@@ -3,9 +3,11 @@
 #' The function is called by the wrapper (hmi). It uses \code{mice} with the method "norm".
 #' @param y_imp A Vector with the variable to impute.
 #' @param X_imp A data.frame with the fixed effects variables.
+#' @param rounding_degrees A numeric vector with the presumed rounding degrees.
 #' @return A n x 1 data.frame with the original and imputed values.
 imp_cont_single <- function(y_imp,
-                      X_imp){
+                      X_imp,
+                      rounding_degrees = c(1, 10, 100, 1000)){
 
 
   #the missing indactor indicates, which values of y are missing.
@@ -16,7 +18,7 @@ imp_cont_single <- function(y_imp,
   X_imp <- cleanup(X_imp)
 
   # standardise the covariates in X (which are numeric and no intercept)
-  X_imp_stand <- stand(X_imp)
+  X_imp_stand <- stand(X_imp, rounding_degrees = rounding_degrees)
 
 
   n <- length(y_imp)
