@@ -24,7 +24,7 @@ head(ex) # e.g. in line 5 there is a NA now.
 ## ----echo = FALSE, message = FALSE, cache = TRUE-------------------------
 library("mice")
 library("hmi")
-#source('C:/Users/Matthias/Documents/hmi/R/hmi_wrapper_2017-10-12.R')
+#source('C:/Users/Matthias/Documents/hmi/R/hmi_wrapper_2017-12-20.R')
 #source('C:/Users/Matthias/Documents/hmi/R/hmi_imp_cont_single_2017-04-11.R')
 
 ## ----eval = FALSE, message = FALSE, cache = TRUE-------------------------
@@ -44,7 +44,7 @@ head(complete(result, 1))
 
 ## ----eval = TRUE, message = TRUE, cache = TRUE---------------------------
 library("lme4")
-lmer(Reaction ~ 1 + Days + (1 + Days | Subject), data = ex)
+lmer(formula = Reaction~1+Days+(1+Days|Subject), data = ex, na.action = na.omit)
 
 ## ----eval = FALSE, message = FALSE, cache = TRUE-------------------------
 #  set.seed(1)
@@ -56,8 +56,8 @@ lmer(Reaction ~ 1 + Days + (1 + Days | Subject), data = ex)
 
 ## ----eval = TRUE, echo = FALSE, results = "hide", message = FALSE, cache = TRUE----
 library("hmi")
-#source('C:/Users/Matthias/Documents/hmi/R/hmi_imp_cont_multi_2017-10-12.R')
-#source('C:/Users/Matthias/Documents/hmi/R/hmi_smallfunctions_2017-10-12.R')
+#source('C:/Users/Matthias/Documents/hmi/R/hmi_imp_cont_multi_2017-10-25.R')
+#source('C:/Users/Matthias/Documents/hmi/R/hmi_smallfunctions_2017-12-28.R')
 ex
 set.seed(1)
 result_multi <- hmi(data = ex, model_formula = Reaction ~ 1 + Days + (1 + Days | Subject), maxit = 1)
@@ -93,7 +93,7 @@ pool(with(data = result_multi, expr = lmer(Reaction ~ 1 + Days + (1 + Days | Sub
 
 ## ----eval = TRUE, echo = FALSE, message = FALSE, cache = TRUE------------
 library("hmi")
-#source('C:/Users/Matthias/Documents/hmi/R/hmi_smallfunctions_2017-10-12.R')
+#source('C:/Users/Matthias/Documents/hmi/R/hmi_smallfunctions_2017-12-28.R')
  my_analysis <- function(complete_data){
   # In this list, you can write all the parameters you are interested in.
   # Those will be averaged.
